(* ==================================================================
The use of this Package is described in "Economic and Financial
Modeling with Mathematica", Hal Varian, editor, TELOS/Springer-Verlag,
1993, ISBN 0-387-97882-8.  $44.95 (Includes 1.44 MS-DOS disk.)  To
place order call 1-800-777-4643 or fax 1-201-348-4505.
=====================================================================*)

BeginPackage["finite`"]

OptimalContract::usage="OptimalContract[n,p,v,t]:
takes as input a number of states n, 
a probability mass function p for the n states, 
a disutility function v(y,t) for producing 
output y with cost t for the agent, and a state cost 
index t[i] for the agent in state i.  It solves the
problem assuming the single crossing property is
satisfied and provides as output the optimal contract
r[i], y[i] for each state i, and the Lagrange 
multiplier lam[i] for the constraint i.  
(Constraints are labelled as described in the text)."

Graphs::usage="Graphs[data,v,t]:
takes as input the data from OptimalContract which is
a 3 n element list for n states (it consists of wages 
r[i], output y[i] and Lagrange multipliers lam[i]).  
It produces two graphs, one which shows how the
wages, outputs and multipliers vary with the state,
and another which shows how rents and profits
vary with the state."


Begin["`Private`"]

OptimalContract[n_Integer:5,pp_:1,vv_:1,tt_:1 ]:=
Block[{lamv,IC,IR,Constraints,Obj,Lagran,Focs,eqns,
sol,v,p,t,r,y,lam},
If[pp===1,p=Binomial[5,#1] (.6^#1) ((1-.6)^(5-#1)) &,
	p=pp];
If[vv===1,v=#1*#1/#2 &,v=vv];
If[tt===1,t=#1 &,t=tt];
	lamv=Table[lam[i],{i,1,n}];
	IC=Table[r[i]-v[y[i],t[i]]-r[i-1]+v[y[i-1],
			t[i]],{i,2,n}];
	IR={r[1]-v[y[1],t[1]]};
	Constraints=Join[IR,IC];
	Obj=Sum[p[i]*(y[i]-r[i]),{i,1,n}];
	Lagran=Obj+lamv.Constraints;
	Focs=Flatten[Table[{D[Lagran,y[i]]==0,
			D[Lagran,r[i]]==0},{i,1,n}]];
	eqns=Join[Focs,Table[Constraints[[i]]==0,
			{i,1,n}]];
	sol=NSolve[eqns,Flatten[Join[
		Table[y[i],{i,1,n}],
		Table[r[i],{i,1,n}],
		Table[lam[i],{i,1,n}]]]];
	Return[sol/.{lam->"lam",r->"r",y->"y"}];];
	
Graphs[data2_List,vv_:1,tt_:1]:=
Module[{v,t,n,ygraph,rgraph,lgraph,pigraph,rentgraph,
lam,y,r,data},
data=data2/.{"lam"->lam,"r"->r,"y"->y};
If[vv===1,v=#1*#1/#2 &,v=vv];
If[tt===1,t=#1 &,t=tt];
n=Floor[Length[Flatten[data]]/3];
ygraph=ListPlot[Flatten[Table[y[i]/.data,
	{i,1,n}]],PlotStyle->{Dashing[{.01,.01}]},
	PlotJoined->True,DisplayFunction->Identity];
rgraph=ListPlot[Flatten[Table[r[i]/.data,
	{i,1,n}]],PlotStyle->{Thickness[.01]},
	PlotJoined->True,
	DisplayFunction->Identity];
lgraph=ListPlot[Flatten[Table[lam[i]/.data,
	{i,1,n}]],PlotStyle->{RGBColor[1,0,0]},
	PlotJoined->True,
	DisplayFunction->Identity];
pigraph=ListPlot[Flatten[Table[y[i]-r[i]/.data,
	{i,1,n}]], 
	PlotJoined->True,
	DisplayFunction->Identity];
rentgraph=ListPlot[Flatten[Table[r[i]-
	v[y[i],t[i]]/.data,{i,1,n}]],
	PlotStyle->Dashing[{.01,.01}],
	PlotJoined->True,DisplayFunction->Identity];
Return[{
Show[lgraph,rgraph,ygraph,AxesLabel->{"type",
	"lambda,rents,output"},
	PlotLabel->"Second-Best Contract",
	DisplayFunction->$DisplayFunction],
Show[pigraph,rentgraph,AxesLabel->{"type",
	"profit, rent"},
	PlotLabel->"Second-Best Rents and Profits",
	DisplayFunction->$DisplayFunction]}]
	];

	
End[]
EndPackage[]


